@@ -4,12 +4,12 @@ import (
 	"encoding/base64"
 	"flag"
 	"fmt"
+	"io"
 	"os"
 
 	"gopkg.in/yaml.v3"
 
 	"github.com/helmfile/vals"
-	"github.com/helmfile/vals/pkg/log"
 )
 
 var (
@@ -90,37 +90,32 @@ func main() {
 		evalCmd := flag.NewFlagSet(CmdEval, flag.ExitOnError)
 		f := evalCmd.String("f", "-", "YAML/JSON file to be evaluated. When set to \"-\", vals reads from STDIN")
 		o := evalCmd.String("o", "yaml", "Output type which is either \"yaml\" or \"json\"")
-		evalCmd.BoolVar(&log.Silent, "s", false, "Silent mode")
+		silent := evalCmd.Bool("s", false, "Silent mode")
 		e := evalCmd.Bool("exclude-secret", false, "Leave secretref+<uri> as-is and only replace ref+<uri>")
 		err := evalCmd.Parse(os.Args[2:])
 		if err != nil {
 			fatal("%v", err)
 		}
 
+		var logOut io.Writer = os.Stderr
+		if *silent {
+			logOut = io.Discard
+		}
+
 		nodes := readNodesOrFail(f)
 
-		var res []yaml.Node
-		for _, node := range nodes {
-			var nodeValue map[string]interface{}
-			err := node.Decode(&nodeValue)
-			if err != nil {
-				fatal("%v", err)
-			}
-			evalResult, err := vals.Eval(nodeValue, vals.Options{ExcludeSecret: *e})
-			if err != nil {
-				fatal("%v", err)
-			}
-			err = node.Encode(evalResult)
-			if err != nil {
-				fatal("%v", err)
-			}
-			res = append(res, node)
+		res, err := vals.EvalNodes(nodes, vals.Options{
+			ExcludeSecret: *e,
+			LogOutput:     logOut,
+		})
+		if err != nil {
+			fatal("%v", err)
 		}
 
 		writeOrFail(o, res)
 	case CmdGet:
 		getCmd := flag.NewFlagSet(CmdGet, flag.ExitOnError)
-		getCmd.BoolVar(&log.Silent, "s", false, "Silent mode")
+		silent := getCmd.Bool("s", false, "Silent mode")
 		err := getCmd.Parse(os.Args[2:])
 		if err != nil {
 			fatal("%v", err)
@@ -131,7 +126,12 @@ func main() {
 			fatal("The first argument of the get command is required")
 		}
 
-		v, err := vals.Get(code)
+		var logOut io.Writer = os.Stderr
+		if *silent {
+			logOut = io.Discard
+		}
+
+		v, err := vals.Get(code, vals.Options{LogOutput: logOut})
 		if err != nil {
 			fatal("%v", err)
 		}
@@ -143,14 +143,29 @@ func main() {
 	case CmdExec:
 		execCmd := flag.NewFlagSet(CmdExec, flag.ExitOnError)
 		f := execCmd.String("f", "", "YAML/JSON file to be loaded to set envvars")
+		streamYAML := execCmd.String("stream-yaml", "", `Reads the specific YAML file or all the YAML files
+stored within the specific directory, evaluate each YAML file,
+joining all the YAML files with "---" lines, and stream the
+result into the stdin of the executed command.
+This is handy when you want to use vals to preprocess
+Kubernetes manifests to kubectl-apply, without writing
+the vals-eval outputs onto the disk, for security reasons.`)
 		err := execCmd.Parse(os.Args[2:])
 		if err != nil {
 			fatal("%v", err)
 		}
 
-		m := readOrFail(f)
+		var m map[string]interface{}
 
-		err = vals.Exec(m, execCmd.Args())
+		if *f != "" {
+			m = readOrFail(f)
+		} else {
+			m = map[string]interface{}{}
+		}
+
+		err = vals.Exec(m, execCmd.Args(), vals.ExecConfig{
+			StreamYAML: *streamYAML,
+		})
 		if err != nil {
 			fatal("%v", err)
 		}
