@@ -1,9 +1,11 @@
 package vals
 
 import (
+	"bytes"
 	"crypto/md5"
 	"errors"
 	"fmt"
+	"io"
 	"net/url"
 	"os"
 	"os/exec"
@@ -16,6 +18,7 @@ import (
 	"github.com/helmfile/vals/pkg/api"
 	"github.com/helmfile/vals/pkg/config"
 	"github.com/helmfile/vals/pkg/expansion"
+	"github.com/helmfile/vals/pkg/log"
 	"github.com/helmfile/vals/pkg/providers/awskms"
 	"github.com/helmfile/vals/pkg/providers/awssecrets"
 	"github.com/helmfile/vals/pkg/providers/azurekeyvault"
@@ -95,6 +98,8 @@ type Runtime struct {
 
 	Options Options
 
+	logger *log.Logger
+
 	m sync.Mutex
 }
 
@@ -107,6 +112,9 @@ func New(opts Options) (*Runtime, error) {
 	r := &Runtime{
 		providers: map[string]api.Provider{},
 		Options:   opts,
+		logger: log.New(log.Config{
+			Output: opts.LogOutput,
+		}),
 	}
 	var err error
 	r.docCache, err = lru.New(cacheSize)
@@ -151,13 +159,13 @@ func (r *Runtime) prepare() (*expansion.ExpandRegexMatch, error) {
 
 		switch scheme {
 		case ProviderVault:
-			p := vault.New(conf)
+			p := vault.New(r.logger, conf)
 			return p, nil
 		case ProviderS3:
 			// ref+s3://foo/bar?region=ap-northeast-1#/baz
 			// 1. GetObject for the bucket foo and key bar
 			// 2. Then extracts the value for key baz(=/foo/bar/baz) from the result from step 1.
-			p := s3.New(conf)
+			p := s3.New(r.logger, conf)
 			return p, nil
 		case ProviderGCS:
 			// vals+gcs://foo/bar?generation=timestamp#/baz
@@ -173,16 +181,16 @@ func (r *Runtime) prepare() (*expansion.ExpandRegexMatch, error) {
 			// ref+awsssm://foo/bar?region=ap-northeast-1#/baz
 			// 1. GetParametersByPath for the prefix /foo/bar
 			// 2. Then extracts the value for key baz(=/foo/bar/baz) from the result from step 1.
-			p := ssm.New(conf)
+			p := ssm.New(r.logger, conf)
 			return p, nil
 		case ProviderSecretsManager:
 			// ref+awssecrets://foo/bar?region=ap-northeast-1#/baz
 			// 1. Get secret for key foo/bar, parse it as yaml
 			// 2. Then extracts the value for key baz) from the result from step 1.
-			p := awssecrets.New(conf)
+			p := awssecrets.New(r.logger, conf)
 			return p, nil
 		case ProviderSOPS:
-			p := sops.New(conf)
+			p := sops.New(r.logger, conf)
 			return p, nil
 		case ProviderEcho:
 			p := echo.New(conf)
@@ -431,6 +439,7 @@ func IgnorePrefix(p string) Option {
 }
 
 type Options struct {
+	LogOutput     io.Writer
 	CacheSize     int
 	ExcludeSecret bool
 }
@@ -452,7 +461,35 @@ func Env(template map[string]interface{}) ([]string, error) {
 	return env, nil
 }
 
-func Exec(template map[string]interface{}, args []string) error {
+type ExecConfig struct {
+	// StreamYAML reads the specific YAML file or all the YAML files
+	// stored within the specific directory, evaluate each YAML file,
+	// joining all the YAML files with "---" lines, and stream the
+	// result into the stdin of the executed command.
+	// This is handy when you want to use vals to preprocess
+	// Kubernetes manifests to kubectl-apply, without writing
+	// the vals-eval outputs onto the disk, for security reasons.
+	StreamYAML string
+
+	Stdout, Stderr io.Writer
+}
+
+func Exec(template map[string]interface{}, args []string, config ...ExecConfig) error {
+	var c ExecConfig
+	if len(config) > 0 {
+		c = config[0]
+	}
+
+	var stdout io.Writer = os.Stdout
+	if c.Stdout != nil {
+		stdout = c.Stdout
+	}
+
+	var stderr io.Writer = os.Stderr
+	if c.Stderr != nil {
+		stderr = c.Stderr
+	}
+
 	if len(args) == 0 {
 		return errors.New("missing args")
 	}
@@ -460,14 +497,50 @@ func Exec(template map[string]interface{}, args []string) error {
 	if err != nil {
 		return err
 	}
+
 	cmd := exec.Command(args[0], args[1:]...)
+
+	if path := c.StreamYAML; path != "" {
+		buf := &bytes.Buffer{}
+
+		if err := streamYAML(path, buf, stderr); err != nil {
+			return err
+		}
+
+		cmd.Stdin = buf
+	} else {
+		cmd.Stdin = os.Stdin
+	}
+
 	cmd.Env = env
-	cmd.Stdin = os.Stdin
-	cmd.Stdout = os.Stdout
-	cmd.Stderr = os.Stderr
+	cmd.Stdout = stdout
+	cmd.Stderr = stderr
+
 	return cmd.Run()
 }
 
+func EvalNodes(nodes []yaml.Node, c Options) ([]yaml.Node, error) {
+	var res []yaml.Node
+	for _, node := range nodes {
+		var nodeValue map[string]interface{}
+		err := node.Decode(&nodeValue)
+		if err != nil {
+			return nil, err
+		}
+		evalResult, err := Eval(nodeValue, c)
+		if err != nil {
+			return nil, err
+		}
+		err = node.Encode(evalResult)
+		if err != nil {
+			return nil, err
+		}
+		res = append(res, node)
+	}
+
+	return res, nil
+}
+
 func Eval(template map[string]interface{}, o ...Options) (map[string]interface{}, error) {
 	opts := Options{}
 	if len(o) > 0 {
@@ -480,8 +553,8 @@ func Eval(template map[string]interface{}, o ...Options) (map[string]interface{}
 	return runtime.Eval(template)
 }
 
-func Get(code string) (string, error) {
-	runtime, err := New(Options{})
+func Get(code string, opts Options) (string, error) {
+	runtime, err := New(opts)
 	if err != nil {
 		return "", err
 	}
@@ -495,6 +568,8 @@ func Load(conf api.StaticConfig, opt ...Option) (map[string]interface{}, error)
 		o(ctx)
 	}
 
+	l := log.New(log.Config{Output: os.Stderr})
+
 	type ValuesProvider struct {
 		ID  []string
 		Get func(api.StaticConfig) map[string]interface{}
@@ -622,7 +697,7 @@ func Load(conf api.StaticConfig, opt ...Option) (map[string]interface{}, error)
 
 	switch tpe {
 	case TypeString:
-		p, err := stringprovider.New(provider)
+		p, err := stringprovider.New(l, provider)
 		if err != nil {
 			return nil, err
 		}
@@ -646,11 +721,11 @@ func Load(conf api.StaticConfig, opt ...Option) (map[string]interface{}, error)
 			return nil, fmt.Errorf("unexpected type: %T", r)
 		}
 	case TypeMap:
-		p, err := stringmapprovider.New(provider)
+		p, err := stringmapprovider.New(l, provider)
 		if err != nil {
 			return nil, err
 		}
-		pp, err := stringprovider.New(provider)
+		pp, err := stringprovider.New(l, provider)
 		if err != nil {
 			return nil, err
 		}
